// Demo: Dual CAN-BUS Shields, Data Pass-through
// Written by: Cory J. Fowler
// January 31st 2014
// This examples the ability of this library to support more than one MCP2515 based CAN interface.


#include "MCP_CAN_lib/mcp_can.h"
#include <SPI.h>

unsigned long rxId;
byte len;
byte rxBuf[8];

#define CAR_CAN_CS 10     // CS of interface where CAN-Bus of car is connected to
#define CAR_CAN_INT 2     // INT of car-interface is connect to this Arduino port
#define NAV_CAN_CS 9      // CS of interface where CAN-Bus of radio is connected to
#define NAV_CAN_INT 3     // INT of radio-interface is connect to this Arduino port

MCP_CAN CAN0(CAR_CAN_CS);
MCP_CAN CAN1(NAV_CAN_CS);

void setup()
{
  Serial.begin(115200);
  
  // init CAN0 bus, baudrate: 125k@8MHz
  if(CAN0.begin(MCP_ANY, CAN_125KBPS, MCP_8MHZ) == CAN_OK){
  Serial.print("CAN0 (CAR): Init OK!\r\n");
  CAN0.setMode(MCP_NORMAL);
  } else Serial.print("CAN0 (CAR): Init Fail!!!\r\n");
  CAN0.enOneShotTX();
  
  // init CAN1 bus, baudrate: 125k@8MHz
  if(CAN1.begin(MCP_ANY, CAN_125KBPS, MCP_8MHZ) == CAN_OK){
  Serial.print("CAN1 (NAV): Init OK!\r\n");
  CAN1.setMode(MCP_NORMAL);
  } else Serial.print("CAN1 (NAV): Init Fail!!!\r\n");
  CAN1.enOneShotTX();
  
  SPI.setClockDivider(SPI_CLOCK_DIV2);         // Set SPI to run at 8MHz (16MHz / 2 = 8 MHz)
}

void loop(){  
  if(!digitalRead(CAR_CAN_INT)){               // If pin 2 is low, read CAN0 receive buffer
    CAN0.readMsgBuf(&rxId, &len, rxBuf);       // Read data: len = data length, buf = data byte(s)
    CAN1.sendMsgBuf(rxId, 0, len, rxBuf);      // Immediately send message out CAN1 interface
  }
  if(!digitalRead(NAV_CAN_INT)){               // If pin 3 is low, read CAN1 receive buffer
    CAN1.readMsgBuf(&rxId, &len, rxBuf);       // Read data: len = data length, buf = data byte(s)
    CAN0.sendMsgBuf(rxId, 0, len, rxBuf);      // Immediately send message out CAN0 interface
  }
}

